@@ -7,6 +7,7 @@ import unittest
 import unittest.mock
 
 import aiohttp
+from aiohttp import helpers
 from aiohttp.client_reqrep import ClientResponse
 
 
@@ -69,9 +70,24 @@ class TestClientResponse(unittest.TestCase):
             '<ClientResponse(http://def-cl-resp.org) [200 Ok]>',
             repr(self.response))
 
+    def test_repr_non_ascii_url(self):
+        response = ClientResponse('get', 'http://fake-host.org/\u03bb')
+        self.assertIn(
+            "<ClientResponse(http://fake-host.org/\\u03bb) [None None]>",
+            repr(response))
+        response.close()
+
+    def test_repr_non_ascii_reason(self):
+        response = ClientResponse('get', 'http://fake-host.org/path')
+        response.reason = '\u03bb'
+        self.assertIn(
+            "<ClientResponse(http://fake-host.org/path) [None \\u03bb]>",
+            repr(response))
+        response.close()
+
     def test_read_and_release_connection(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result(b'payload')
             return fut
         content = self.response.content = unittest.mock.Mock()
@@ -83,7 +99,7 @@ class TestClientResponse(unittest.TestCase):
 
     def test_read_and_release_connection_with_error(self):
         content = self.response.content = unittest.mock.Mock()
-        content.read.return_value = asyncio.Future(loop=self.loop)
+        content.read.return_value = helpers.create_future(self.loop)
         content.read.return_value.set_exception(ValueError)
 
         self.assertRaises(
@@ -92,7 +108,7 @@ class TestClientResponse(unittest.TestCase):
         self.assertTrue(self.response._closed)
 
     def test_release(self):
-        fut = asyncio.Future(loop=self.loop)
+        fut = helpers.create_future(self.loop)
         fut.set_result(b'')
         content = self.response.content = unittest.mock.Mock()
         content.readany.return_value = fut
@@ -103,7 +119,7 @@ class TestClientResponse(unittest.TestCase):
     def test_read_decode_deprecated(self):
         self.response._content = b'data'
         self.response.json = unittest.mock.Mock()
-        self.response.json.return_value = asyncio.Future(loop=self.loop)
+        self.response.json.return_value = helpers.create_future(self.loop)
         self.response.json.return_value.set_result('json')
 
         with self.assertWarns(DeprecationWarning):
@@ -113,7 +129,7 @@ class TestClientResponse(unittest.TestCase):
 
     def test_text(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
             return fut
         self.response.headers = {
@@ -127,7 +143,7 @@ class TestClientResponse(unittest.TestCase):
 
     def test_text_custom_encoding(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
             return fut
         self.response.headers = {
@@ -144,7 +160,7 @@ class TestClientResponse(unittest.TestCase):
 
     def test_text_detect_encoding(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
             return fut
         self.response.headers = {'CONTENT-TYPE': 'application/json'}
@@ -158,7 +174,7 @@ class TestClientResponse(unittest.TestCase):
 
     def test_text_after_read(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
             return fut
         self.response.headers = {
@@ -172,7 +188,7 @@ class TestClientResponse(unittest.TestCase):
 
     def test_json(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
             return fut
         self.response.headers = {
@@ -209,7 +225,7 @@ class TestClientResponse(unittest.TestCase):
 
     def test_json_override_encoding(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
             return fut
         self.response.headers = {
@@ -226,7 +242,7 @@ class TestClientResponse(unittest.TestCase):
 
     def test_json_detect_encoding(self):
         def side_effect(*args, **kwargs):
-            fut = asyncio.Future(loop=self.loop)
+            fut = helpers.create_future(self.loop)
             fut.set_result('{"тест": "пройден"}'.encode('cp1251'))
             return fut
         self.response.headers = {'CONTENT-TYPE': 'application/json'}
@@ -258,3 +274,16 @@ class TestClientResponse(unittest.TestCase):
         with self.assertWarns(DeprecationWarning):
             self.response.close(force=False)
         self.assertIsNone(self.response._connection)
+
+    def test_raise_for_status_2xx(self):
+        self.response.status = 200
+        self.response.reason = 'OK'
+        self.response.raise_for_status()  # should not raise
+
+    def test_raise_for_status_4xx(self):
+        self.response.status = 409
+        self.response.reason = 'CONFLICT'
+        with self.assertRaises(aiohttp.HttpProcessingError) as cm:
+            self.response.raise_for_status()
+        self.assertEqual(str(cm.exception.code), '409')
+        self.assertEqual(str(cm.exception.message), "CONFLICT")
