@@ -1,8 +1,8 @@
 import collections
 import pytest
 import re
+from multidict import CIMultiDict
 from unittest import mock
-from aiohttp.multidict import CIMultiDict
 from aiohttp.web import Request
 from aiohttp.protocol import RawRequestMessage, HttpVersion11
 
@@ -31,7 +31,7 @@ def request(buf):
     app = mock.Mock()
     app._debug = False
     app.on_response_prepare = signals.Signal(app)
-    message = RawRequestMessage(method, path, HttpVersion11, headers,
+    message = RawRequestMessage(method, path, HttpVersion11, headers, [],
                                 False, False)
     req = Request(app, message, payload,
                   transport, reader, writer)
@@ -57,7 +57,6 @@ def test_HTTPOk(buf, request):
     assert re.match(('HTTP/1.1 200 OK\r\n'
                      'CONTENT-TYPE: text/plain; charset=utf-8\r\n'
                      'CONTENT-LENGTH: 7\r\n'
-                     'CONNECTION: keep-alive\r\n'
                      'DATE: .+\r\n'
                      'SERVER: .+\r\n\r\n'
                      '200: OK'), txt)
@@ -95,7 +94,6 @@ def test_HTTPFound(buf, request):
                     'CONTENT-TYPE: text/plain; charset=utf-8\r\n'
                     'CONTENT-LENGTH: 10\r\n'
                     'LOCATION: /redirect\r\n'
-                    'CONNECTION: keep-alive\r\n'
                     'DATE: .+\r\n'
                     'SERVER: .+\r\n\r\n'
                     '302: Found', txt)
@@ -122,7 +120,6 @@ def test_HTTPMethodNotAllowed(buf, request):
                     'CONTENT-TYPE: text/plain; charset=utf-8\r\n'
                     'CONTENT-LENGTH: 23\r\n'
                     'ALLOW: POST,PUT\r\n'
-                    'CONNECTION: keep-alive\r\n'
                     'DATE: .+\r\n'
                     'SERVER: .+\r\n\r\n'
                     '405: Method Not Allowed', txt)
@@ -166,3 +163,10 @@ def test_empty_body_205():
 def test_empty_body_304():
     resp = web.HTTPNoContent()
     resp.body is None
+
+
+def test_link_header_451(buf, request):
+    resp = web.HTTPUnavailableForLegalReasons(link='http://warning.or.kr/')
+
+    assert 'http://warning.or.kr/' == resp.link
+    assert '<http://warning.or.kr/>; rel="blocked-by"' == resp.headers['Link']
